@@ -20,21 +20,13 @@ module Test_Symbol
 
     def assert_not_interned_error(obj, meth, name, msg = nil, &block)
       e = assert_raise(NameError, msg) {obj.__send__(meth, name, &block)}
-      if Symbol === name
-        assert_not_pinneddown(name, msg)
-      else
-        assert_not_interned(name, msg)
-      end
+      assert_not_pinneddown(name, msg)
       e
     end
 
     def assert_not_interned_false(obj, meth, name, msg = nil)
       assert_not_send([obj, meth, name], msg)
-      if Symbol === name
-        assert_not_pinneddown(name, msg)
-      else
-        assert_not_interned(name, msg)
-      end
+      assert_not_pinneddown(name, msg)
     end
 
     Feature5072 = '[ruby-core:38367]'
@@ -490,5 +482,14 @@ module Test_Symbol
         foo.call(name.to_sym => 42)
       end
     end
+
+    def test_iv_get
+      obj = Object.new
+      assert_warning(/not initialized/) do
+        assert_no_immortal_symbol_created("rb_iv_get") do |name|
+          Bug::Symbol.iv_get(obj, name)
+        end
+      end
+    end
   end
 end
